
%%
% Updated 04/16/2019
% Research with Steve Engel and Ping Chen
%
% Check EAP for \theta=(a, b, c, \beta, \lambda=\gamma)
% Range of the parameters in $\theta$
% a in the range of 46 to 50
% b in the range of 1 to 3
% c in the range of 0.01 to 0.1
% beta=2 (fixed)
% lambda=gamma=0.05 (fixed)
% t in the range of 1 to 120
% orientation in 41:0.25:55 (denoted by x)

clear;
clc;
close all;

%% Set up true values
beta=2;
lambda=0.05;
gamma=0.05;
t_block=(1:3:120)';                                            % 40 trials per block (i.e., T=40)
x_block=(41:0.25:55)';                                      % 57 orientations in total (i.e., R=57)
nblocks=5;                                                         % J=5
t=repmat(t_block,nblocks,1);                            % replicate the block 5 times
nparameters=3;

%% Generate MC samples and prior values for theta
% Because each parameters in $\theta$ have their own distribution, we have to sample them separately from their own prior
a_mc=46:0.5:50;                                                % 9 values
b_mc=1:0.25:3;                                                  % 9 values
c_mc=0.01:0.01:0.1;                                          % 10 values
[theta_mc,theta_true,prior_value]=generate_theta_mc(a_mc,b_mc,c_mc,nparameters);

%% Simulate responses to the trials (with all possible orientations) at different time points for 810 persons in advance
nstimuli=length(x_block);
ntimepoints=length(t_block);
total_trial_length=length(t);
[nexaminees,~]=size(theta_true);                      % 9*9*10=810
response_matrix=cell(nexaminees,1);               % each examinee has a response matrix
% randnumber=cell(nexaminees,1);
load randnumber.mat;
for i=1:nexaminees
    % randnumber{i,1}=rand(nstimuli,total_test_length);
    alpha=theta_true(i,1)-theta_true(i,2)*exp(-theta_true(i,3)*t_block);         % alpha is of the same size as t_block
    correct_prob=repmat(gamma+(1-gamma-lambda)./(1+exp(-beta*(repmat(x_block,1,ntimepoints)-repmat(alpha',nstimuli,1)))),1,nblocks);
    response_matrix{i,1}=(correct_prob>=randnumber{i,1});                        % size of nstimuli*total_test_length
end
% save randnumber.mat randnumber;

%% Prepare P(Y^k=y|theta^(m)) for function entropy_selection
IRFs_Next=IRFs_Next_Computation(x_block,theta_mc,t_block,beta,gamma,lambda);

%% Simulate the adaptive process using ENTROPY, RANDOM, UP&DOWN and P=0.5 item selection strategies and compare their performance
nmethods=4;
total_sample=length(prior_value);

ptheta_entropy=zeros(nexaminees,nparameters);             % with same format as theta_true
ptheta_random=zeros(nexaminees,nparameters);
ptheta_updown=zeros(nexaminees,nparameters);
ptheta_p05=zeros(nexaminees,nparameters);
eap_se_entropy=zeros(nexaminees,nparameters);
eap_se_random=zeros(nexaminees,nparameters);
eap_se_updown=zeros(nexaminees,nparameters);
eap_se_p05=zeros(nexaminees,nparameters);
time_used_entropy=zeros(nexaminees,total_trial_length);
time_used_random=zeros(nexaminees,total_trial_length);
time_used_updown=zeros(nexaminees,total_trial_length);
time_used_p05=zeros(nexaminees,total_trial_length);

number_being_answered=zeros(nstimuli,nmethods);        % number of times each stimulus gets selected
id_pattern=cell(nexaminees,1);                                           % record IDs of all stimuli each examinee has responded
response_pattern=cell(nexaminees,1);

for i=1:nexaminees
    disp(['The ',num2str(i),'th examinee!']);
    u=zeros(total_trial_length,nmethods);
    v=zeros(total_trial_length,nmethods);
    for method_ID=1:nmethods
        trial_length=0;
        flag=1;
        while (flag==1)
            switch method_ID
                case 1                                                 % ENTROPY
                    if (trial_length==0)
                        tic;
                        stimulus_id=fix(rand*(34-25))+25;        % randomly select the first stimulus from Stimuli 25 to 33 located in the middle
                        time_used_entropy(i,trial_length+1)=toc;
                        ilikelihood=ones(total_sample,1);
                    else
                        tic;
                        [likelihood,stimulus_id]=entropy_selection(x_block,u(:,method_ID),v(:,method_ID),trial_length,prior_value,ilikelihood,IRFs_Next,t_block);
                        time_used_entropy(i,trial_length+1)=toc;
                        ilikelihood=likelihood;
                    end
                case 2                                                 % RANDOM
                    tic;
                    stimulus_id=fix(nstimuli*rand+1);
                    time_used_random(i,trial_length+1)=toc;
                case 3                                                 % UP&DOWN
                    tic;
                    if (fix(trial_length/ntimepoints)==0)         % first block
                        stimulus_id=fix(nstimuli*rand+1);
                    else
                        previous_item_id=u(trial_length+1-ntimepoints,method_ID);            % ID of the item in the previous block
                        previous_response=v(trial_length+1-ntimepoints,method_ID);
                        if (previous_response==1)                    % if answer correctly, present a more difficult one
                            if (previous_item_id==1)
                                stimulus_id=previous_item_id;
                            else
                                stimulus_id=previous_item_id-1;
                            end
                        else                                                        % if answer incorrectly, administer an easier one
                            if (previous_item_id==nstimuli)
                                stimulus_id=previous_item_id;
                            else
                                stimulus_id=previous_item_id+1;
                            end
                        end
                    end
                    time_used_updown(i,trial_length+1)=toc;
                case 4                                                 % select next stimulus that makes P(Y=1) nearest to 0.5
%                     if (trial_length==0)
%                         tic;
%                         stimulus_id=fix(rand*(34-25))+25;        % randomly select the first stimulus from Stimuli 25 to 33 located in the middle
%                         time_used_p05(i,trial_length+1)=toc;
%                         ilikelihood=ones(total_sample,1);                        
%                     else
%                         tic;
%                         [likelihood,stimulus_id]=expected_probability_05_selection(x_block,u(:,method_ID),v(:,method_ID),trial_length,ilikelihood,IRFs_Next,t_block);
%                         time_used_p05(i,trial_length+1)=toc;
%                         ilikelihood=likelihood;                                           
%                     end
                    if (trial_length==0)
                        tic;
                        stimulus_id=fix(rand*(34-25))+25;
                        time_used_p05(i,trial_length+1)=toc;
                    else
                        tic;
                        [temp_estimate,~]=EAP_person(theta_mc,v(1:trial_length,method_ID),x_block(u(1:trial_length,method_ID),1),t(1:trial_length,1),beta,gamma,lambda);
                        stimulus_id=probability_05_selection(temp_estimate,x_block,t(trial_length+1,1),beta,gamma,lambda);
                        time_used_p05(i,trial_length+1)=toc;
                    end
            end
            number_being_answered(stimulus_id,method_ID)=number_being_answered(stimulus_id,method_ID)+1;
            trial_length=trial_length+1;
            u(trial_length,method_ID)=stimulus_id;
            v(trial_length,method_ID)=response_matrix{i,1}(stimulus_id,trial_length);
            if (trial_length>=total_trial_length)
                flag=0;
            end
        end
    
        % Estimate ability vector using EAP method
        if (method_ID==1)
            [ptheta_entropy(i,:),eap_se_entropy(i,:)]=EAP_person(theta_mc,v(:,method_ID),x_block(u(:,method_ID),1),t,beta,gamma,lambda);
        elseif (method_ID==2)
            [ptheta_random(i,:),eap_se_random(i,:)]=EAP_person(theta_mc,v(:,method_ID),x_block(u(:,method_ID),1),t,beta,gamma,lambda);
        elseif (method_ID==3)
            [ptheta_updown(i,:),eap_se_updown(i,:)]=EAP_person(theta_mc,v(:,method_ID),x_block(u(:,method_ID),1),t,beta,gamma,lambda);
        else
            [ptheta_p05(i,:),eap_se_p05(i,:)]=EAP_person(theta_mc,v(:,method_ID),x_block(u(:,method_ID),1),t,beta,gamma,lambda);
        end
    end
    id_pattern{i,1}=u;
    response_pattern{i,1}=v;
end

% Compute the area
area=zeros(nexaminees,nmethods);
for i=1:nexaminees
    for j=1:nmethods
        if (j==1)
            area(i,j)=integral(@(t) integral_fun(ptheta_entropy(i,:),theta_true(i,:),t),min(t_block),max(t_block));
        elseif (j==2)
            area(i,j)=integral(@(t) integral_fun(ptheta_random(i,:),theta_true(i,:),t),min(t_block),max(t_block));
        elseif (j==3)
            area(i,j)=integral(@(t) integral_fun(ptheta_updown(i,:),theta_true(i,:),t),min(t_block),max(t_block));
        else
            area(i,j)=integral(@(t) integral_fun(ptheta_p05(i,:),theta_true(i,:),t),min(t_block),max(t_block));
        end
    end
end

% Evaluate the performance of adaptive designs and random design
[theta_MSE,theta_BIAS,theta_RBIAS,mean_ED,mean_SE,mean_area,mean_time,chi_square]=evaluate_performance(theta_true,ptheta_entropy,ptheta_random,ptheta_updown,ptheta_p05,eap_se_entropy,eap_se_random,eap_se_updown,eap_se_p05,area,time_used_entropy,time_used_random,time_used_updown,time_used_p05,number_being_answered,nstimuli);

% Output Represented by Figures
% Draw_Plot(theta_true,ptheta_entropy,ptheta_random,ptheta_updown,ptheta_p05,eap_se_entropy,eap_se_random,eap_se_updown,eap_se_p05,t_block,area);

% Save some important results
save important_result_condition1.mat theta_MSE theta_BIAS theta_RBIAS mean_ED mean_SE mean_area mean_time chi_square;
save interim_result_condition1.mat theta_true ptheta_entropy ptheta_random ptheta_updown ptheta_p05 eap_se_entropy eap_se_random eap_se_updown eap_se_p05 time_used_entropy time_used_random time_used_updown time_used_p05 number_being_answered id_pattern response_pattern area;

